package client.game;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.KeyEvent;
import java.util.Random;

import net.packet.Packet;

import org.lwjgl.LWJGLException;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.Display;

import shared.TextFormatter;
import client.Processor;
import client.game.PlayerManager.RemotePlayer;
import client.game.block.TileBlock;
import client.game.block.TileBlockManager;
import client.net.Connection;

import com.jpct.twod.GLFont;
import com.jpct.twod.TwoD;
import com.threed.jpct.Camera;
import com.threed.jpct.Config;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.IRenderer;
import com.threed.jpct.Interact2D;
import com.threed.jpct.Lights;
import com.threed.jpct.Loader;
import com.threed.jpct.Logger;
import com.threed.jpct.Matrix;
import com.threed.jpct.Object3D;
import com.threed.jpct.PolygonManager;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureInfo;
import com.threed.jpct.TextureManager;
import com.threed.jpct.World;
import com.threed.jpct.util.KeyMapper;
import com.threed.jpct.util.KeyState;
import com.threed.jpct.util.Light;

public class Application {

	private FrameBuffer fb;
	private World world;
	private KeyMapper keyMapper;
	private Processor processor;
	private Camera camera;
	private TwoD twod;
	public LocalPlayer player;
	private Cache cache;
	private Connection connection;
	public boolean loggedIn = false;
	public int reconnectionAttempt = 0;
	private long lastReconnectionAttempt;
	private String username;
	private String hostname;// = "localhost";

	private boolean forward;
	private boolean backward;
	private boolean left;
	private boolean right;
	@SuppressWarnings("unused")
	private boolean lookUp;
	@SuppressWarnings("unused")
	private boolean lookDown;
	private boolean firstPerson;

	private final GLFont chatbox = new GLFont(new Font("Consoles", Font.PLAIN, 12));
	private final GLFont debug = new GLFont(new Font("Consoles", Font.PLAIN, 10));
	private Color foggingColor = new Color(195, 210, 230);

	private long lastTextUpdate = 0L;
	private String debugText = "";
	
	private int textureId = 1;
	private long lastTextureScroll = 0L;
	private int sandboxMode = -1; // -1 = none, 0 = block, 1 = tile
	private Object3D nextBlock;
	private Object3D nextTile;

	private TileBlockManager tileBlockManager;
	private PlayerManager playerManager;

	private long playerInteractionThrottler = 0L;

	private boolean wireFrame = false;

	public int coins = 0;
	public int currentHealth = 0;
	public int cachedHealth = 0;

	public Application(String username, String hostname) {
		this.username = username;
		this.hostname = hostname;

		Config.glVerbose = true;
		Logger.setLogLevel(Logger.LL_ONLY_ERRORS);
		Logger.setOnError(Logger.ON_ERROR_THROW_EXCEPTION);
		Config.glAvoidTextureCopies = true;
		Config.maxPolysVisible = 10000;
		Config.glColorDepth = 24;
		Config.useMultipleThreads = Runtime.getRuntime().availableProcessors() > 1;
		Config.glTrilinear = false;
		Config.texelFilter = false;
		Config.glTriangleStrips = true;
		Config.autoBuild = true;
		Config.glWindowName = "Multiplayer Sandbox";
		Config.farPlane = 300;
		Config.glFullscreen = false;
		try {
			fb = new FrameBuffer(640, 480, FrameBuffer.SAMPLINGMODE_NORMAL);
			fb.disableRenderer(IRenderer.RENDERER_SOFTWARE);
			fb.enableRenderer(IRenderer.RENDERER_OPENGL);
			twod = new TwoD("./client_data/TwoD/");
			keyMapper = new KeyMapper();

			world = new World();
			this.camera = world.getCamera();
			camera.setFOV(1.5f);
			if (Config.glTriangleStrips) {
				world.createTriangleStrips();
			}
			Config.lightMul = 1;
			world.setFoggingMode(World.FOGGING_PER_PIXEL);
			world.setFogging(World.FOGGING_ENABLED);
			world.getLights().setRGBScale(Lights.RGB_SCALE_2X);
			world.setClippingPlanes(1, Config.farPlane);
			Config.tuneForOutdoor(); // Setup some optimizations for outdoor rendering

			this.cache = new Cache();
			this.tileBlockManager = new TileBlockManager(world, cache);
			nextBlock = cache.getTileBlockModel(true, textureId);
			nextBlock.compile();
			world.addObject(nextBlock);
			nextTile = cache.getTileBlockModel(false, textureId);
			nextTile.compile();
			world.addObject(nextTile);
			this.playerManager = new PlayerManager(world, cache);

			Object3D terrain = Object3D.mergeAll(Loader.loadOBJ("./client_data/flat_terrain.obj", null, .20f)); // .10f = 410x410
			terrain.rotateX(-(float) Math.PI / 2f);
			terrain.rotateMesh();
			terrain.setRotationMatrix(new Matrix());
			terrain.setCollisionMode(Object3D.COLLISION_CHECK_OTHERS);
			terrain.enableLazyTransformations();
			//terrain.setLighting(Object3D.LIGHTING_NO_LIGHTS);
			int terrain_texture_id = TextureManager.getInstance().getTextureID("block-0");
			float[] bb = terrain.getMesh().getBoundingBox();
			float minX = bb[0];
			float maxX = bb[1];
			float minZ = bb[4];
			float maxZ = bb[5];
			float dx = maxX - minX;
			float dz = maxZ - minZ;
			dx /= (160f);
			dz /= (160f);
			PolygonManager pm = terrain.getPolygonManager();
			for (int i = 0; i < pm.getMaxPolygonID(); i++) {
				SimpleVector v0 = pm.getTransformedVertex(i, 0);
				SimpleVector v1 = pm.getTransformedVertex(i, 1);
				SimpleVector v2 = pm.getTransformedVertex(i, 2);
				TextureInfo ti = new TextureInfo(terrain_texture_id, v0.x / dx, v0.z / dz, v1.x / dx, v1.z / dz, v2.x / dx, v2.z / dz);
				pm.setPolygonTexture(i, ti);
			}
			//terrain.setAdditionalColor(new Color(1, 140, 10));
			terrain.compileAndStrip();
			world.addObject(terrain);
			System.out.println("Ground Level: " + terrain.getTransformedCenter().y);

			letThereBeLight(terrain);
			toggleLighting();
			toggleLighting();

			this.player = new LocalPlayer(world, cache.getRangerModel(), terrain.getTransformedCenter().y);
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Error initializing game! Shutting down..");
			System.exit(0);
		}

		try {
			Mouse.create();
		} catch(LWJGLException e) {
			e.printStackTrace();
			System.out.println("Unable to create mouse instance! Shutting down..");
			System.exit(0);
		}

		new Thread(this.processor = new Processor(this)).run();
		World.setDefaultThread(Thread.currentThread());
	}

	public void cycle(final float delta) {

		//if (lastDataExchange <= System.currentTimeMillis() - 10000) {
		//	close();
		//}

		int mouseX = Mouse.getX();
		int mouseY = (fb.getOutputHeight() - Mouse.getY());

		while (Mouse.next()) {
			/**
			 * The inventory interface buttons
			 */
			mouseX = Mouse.getX();
			mouseY = (fb.getOutputHeight() - Mouse.getY());

			if (mouseX >= Display.getDisplayMode().getWidth() - 46 && mouseY >= Display.getDisplayMode().getHeight() - 42  && Mouse.getEventButton() > -1 && !Mouse.getEventButtonState()) {
				Packet packet = new Packet(9);
				packet.putInt(0); // logout button id
				connection.sendPacket(packet);
			}

			if (sandboxMode != -1) {
				if (Mouse.getEventButtonState() && Mouse.getEventButton() == 0) {
					buildBlock(false);
				}
				if (Mouse.getEventButtonState() && Mouse.getEventButton() == 1) {
					buildBlock(true);
				}
				int dWheel = Mouse.getDWheel();
				if (lastTextureScroll <= System.currentTimeMillis() - 250) {
					lastTextureScroll = System.currentTimeMillis();
					if (dWheel < 0) {
						if (textureId > 0) {
							textureId -= 1;
							nextBlock.setTexture("block-"+textureId);
							nextTile.setTexture("block-"+textureId);
						}
					} else if (dWheel > 0) {
						if (textureId < 15) {
							textureId += 1;
							nextBlock.setTexture("block-"+textureId);
							nextTile.setTexture("block-"+textureId);
						}
					}
				}
			}
		}

		checkKeys(delta);

		player.animate();
		player.cycle(world);

		camera.setPositionToCenter(player);
		camera.align(player); // align the camera with the player
		if (!firstPerson) {
			camera.rotateCameraX((float) Math.toRadians(20));
			camera.moveCamera(Camera.CAMERA_MOVEOUT, 25);
		} else {
			camera.rotateCameraX((float) Math.toRadians(10));
			camera.moveCamera(Camera.CAMERA_MOVEUP, 3f); // force specific camera height
			camera.moveCamera(Camera.CAMERA_MOVEIN, 1); // move orb to the front of the player's face
		}

		playerManager.update();

		if (renderNextBlockUpdatedAt <= System.currentTimeMillis() - 100) {
			renderNextBlockUpdatedAt = System.currentTimeMillis();
			renderNextBlock();
		}

		tileBlockManager.update(player);
	}

	private final Color MATRIX_GREEN = new Color(0x00CC00);

	// Render to screen
	public void render(int fps) {
		if (!Display.isActive()) {
			fb.clear(Color.BLACK);
			debug.drawString(fb, "IDLE - RENDERING HAS BEEN DISABLED UNTIL YOU RETURN :)", 5, 30, Color.WHITE);
			fb.update();
			fb.displayGLOnly();
			return;
		}
		if (loggedIn) {
			if (!wireFrame) {
				if (sun.isEnabled()) {
					fb.clear(foggingColor);
					twod.drawImage(fb, "skybox", 0, 0, false);
				} else {
					fb.clear(Color.BLACK);
					twod.drawImage(fb, "starbox", 0, 0, false);
				}
				world.renderScene(fb);
				world.draw(fb);
			} else {
				fb.clear(Color.BLACK);
				world.renderScene(fb);
				world.drawWireframe(fb, MATRIX_GREEN);
			}
			renderUI(fps);
		} else {
			fb.clear(Color.BLACK);
			if (reconnectionAttempt > 5) {
				chatbox.drawString(fb, "Unable to connect to server @ " + hostname + ":" + 43594, 5, 30, Color.WHITE);
			} else {
				chatbox.drawString(fb, "Establishing connection with "+Config.glWindowName+" server.. Try #"+reconnectionAttempt+" / 5", 5, 30, Color.ORANGE);
			}
			if (lastReconnectionAttempt <= System.currentTimeMillis() - 5000) {
				lastReconnectionAttempt = System.currentTimeMillis();
				sendLoginRequest("password");
			}
		}
		fb.update();
		fb.displayGLOnly();
	}

	private String[] messages = {"","","","","","","","","",""};
	private long consoleInputThrottler = 0L;
	private String consoleInput = "";
	private boolean consoleInputFlag = false;
	private void renderUI(final int fps) {
		if (lastTextUpdate <= System.currentTimeMillis() - 1000) {
			lastTextUpdate = System.currentTimeMillis();
			debugText = "FPS: " + fps + ", Memory Usage: " + ((Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) >> 20) + "MB";
		}
		debug.drawShadowedString(fb, debugText, 5, 15, Color.WHITE);
		debug.drawShadowedString(fb, "Cached Props: " + tileBlockManager.getCachedTileBlocks(), 5, 30, Color.YELLOW);
		debug.drawShadowedString(fb, "Visible Props: " + tileBlockManager.getTileBlocksInView(), 5, 42, Color.YELLOW);
		debug.drawShadowedString(fb, "Players Online: " + playerManager.getPlayersOnline(), 5, 72, Color.YELLOW);

		for (int i = 0; i < messages.length; i++) { // draw console messages in reverse order
			chatbox.drawString(fb, messages[messages.length - i - 1], 4, Display.getDisplayMode().getHeight() - 21 - (i * 13), Color.BLACK);
		}
		chatbox.drawString(fb, consoleInputFlag ? consoleInput + "*" : "Press [TAB] to enter a message.", 4, Display.getDisplayMode().getHeight() - 5, Color.BLUE);

		// draw logout button
		twod.drawImage(fb, "logout", Display.getDisplayMode().getWidth() - 46, Display.getDisplayMode().getHeight() - 42, true);

		// TODO : add a lot more areas! this is an original feature. and fits nicely with the
		// open terrain setup.
		if (player.getX() > 0 && player.getX() < 0 && player.getZ() > 0 && player.getZ() < 0) {
		}

		twod.drawImage(fb, "focuspoint", Mouse.getX() - 8, (fb.getOutputHeight() - Mouse.getY()) - 8, true);
	}

	@SuppressWarnings("unused")
	private String getBlockType() {
		switch (textureId) {
		case 0: return "Grass";
		case 1: return "Dirt";
		case 2: return "Sand";
		case 3: return "Gravel";
		case 4: return "Stone";
		case 5: return "Cobblestone";
		case 6: return "Brick";
		case 7: return "Snow";
		case 8: return "Clay";
		case 9: return "Iron Ore";
		case 10: return "Coal Ore";
		case 11: return "Gold Ore";
		case 12: return "Acacia Wood";
		case 13: return "Big Oak Wood";
		case 14: return "Oak Wood";
		case 15: return "Spruce Wood";
		default: return "?";
		}
	}

	public void addMessage(String message) {
		for (int i = 0; i < messages.length; i++) {
			if (i == messages.length - 1) {
				messages[i] = message;
			} else {
				messages[i] = messages[i + 1];
			}
		}
	}

	private void checkKeys(float delta) {
		KeyState ks = null;
		while ((ks = keyMapper.poll()) != KeyState.NONE) {
			if (ks.getKeyCode() == KeyEvent.VK_ESCAPE && ks.getState()) {
				processor.terminate();
			}
			if (ks.getKeyCode() == KeyEvent.VK_TAB && ks.getState()) { // toggle chatbox input
				consoleInputFlag = !consoleInputFlag;
			}
			if (consoleInputFlag) { // chatbox
				if (ks.getKeyCode() == KeyEvent.VK_ENTER && consoleInputThrottler <= System.currentTimeMillis() - 1000) {
					consoleInputThrottler = System.currentTimeMillis();
					consoleInputFlag = false;
					/**
					 * Handle client-side command
					 */
					if (consoleInput.startsWith("::")) {
						String command = consoleInput.substring(2);
						if (command.equalsIgnoreCase("wire")) {
							wireFrame = !wireFrame;
						}
						if (command.equalsIgnoreCase("tod")) {
							toggleLighting();
						}
					}
					Packet packet = new Packet(4);
					packet.putString(consoleInput);
					connection.sendPacket(packet);
					consoleInput = "";
				} else {
					consoleInput = TextFormatter.updateText(consoleInput, ks.getKeyCode(), ks.getChar(), 75);
				}
			} else { // game controls
				int player_interaction_hot_key = -1;
				if (ks.getKeyCode() == KeyEvent.VK_1 && ks.getState()) {
					player_interaction_hot_key = 2;
				}
				if (ks.getKeyCode() == KeyEvent.VK_2 && ks.getState()) {
					player_interaction_hot_key = 4;
				}
				if (ks.getKeyCode() == KeyEvent.VK_3 && ks.getState()) {
					player_interaction_hot_key = 8;
				}
				if (player_interaction_hot_key > 0) {
					if (playerInteractionThrottler <= System.currentTimeMillis() - 1000) {
						playerInteractionThrottler = System.currentTimeMillis();
						RemotePlayer closestPlayer = playerManager.getClosestRemotePlayer(player.getTransformedCenter());
						if (closestPlayer != null) {
							Packet packet = new Packet(8);
							packet.putInt(closestPlayer.getUid());
							packet.putInt(player_interaction_hot_key);
							connection.sendPacket(packet);
							System.out.println("[client debug]Attacking remote player (uid="+closestPlayer.getUid());
						}
					}
				}
				if (ks.getKeyCode() == KeyEvent.VK_B && ks.getState()) {
					sandboxMode = sandboxMode == 0 ? -1 : 0;
				}
				if (ks.getKeyCode() == KeyEvent.VK_T && ks.getState()) {
					sandboxMode = sandboxMode == 1 ? -1 : 1;
				}
				if (ks.getKeyCode() == KeyEvent.VK_SPACE && ks.getState()) {
					player.jump();
				}
				if (ks.getKeyCode() == KeyEvent.VK_C && ks.getState()) {
					firstPerson = !firstPerson;
				}
				if (ks.getKeyCode() == KeyEvent.VK_P && ks.getState()) {
					player.printPosition();
				}
				if (ks.getKeyCode() == KeyEvent.VK_W || ks.getKeyCode() == KeyEvent.VK_UP) {
					forward = ks.getState();
				}
				if (ks.getKeyCode() == KeyEvent.VK_S || ks.getKeyCode() == KeyEvent.VK_DOWN) {
					backward = ks.getState();
				}
				if (ks.getKeyCode() == KeyEvent.VK_A || ks.getKeyCode() == KeyEvent.VK_LEFT) {
					left = ks.getState();
				}
				if (ks.getKeyCode() == KeyEvent.VK_D || ks.getKeyCode() == KeyEvent.VK_RIGHT) {
					right = ks.getState();
				}
			}
		}
		move(delta);
	}

	private void move(float delta) {
		if (delta == 0) {
			return;
		}
		if (forward) {
			player.moveForward();
		}
		if (backward) {
			player.moveBackward();
		}
		if (left) {
			player.rotateLeft(firstPerson);
		}
		if (right) {
			player.rotateRight(firstPerson);
		}
		//if (lookUp || lookDown)) {
		// TODO override default camera settings with this...
		// we want the player to be able to look up and down while building
		// and viewing creations..
		//}
		if (player.hasPositionUpdateFlag()) {//forward || backward || left || right) {
			Packet packet = new Packet(2);
			packet.putFloat(player.getTransformedCenter().x);
			packet.putFloat(player.getTransformedCenter().y);
			packet.putFloat(player.getTransformedCenter().z);
			packet.putFloat(player.rotationY);
			connection.sendPacket(packet);
		}
	}

	public void sendLoginRequest(String password) {
		if (loggedIn || reconnectionAttempt > 5) {
			return;
		}
		reconnectionAttempt++;
		if (connection == null) {
			this.connection = new Connection(this);
		}
		connection.connect(hostname, 43594);

		Packet packet = new Packet(1);
		packet.putInt(new Random().nextBoolean() ? 36 : 42);
		packet.putString(username);
		packet.putString(password);
		connection.sendPacket(packet);
	}

	private Light sun;
	private void letThereBeLight(Object3D terrain) {
		sun = new Light(world);
		sun.setIntensity(90, 90, 90);
		sun.setAttenuation(-1);
		sun.setPosition(new SimpleVector(terrain.getTransformedCenter().x, terrain.getTransformedCenter().y - 500, terrain.getTransformedCenter().z));

		TextureManager.getInstance().addTexture("sun", new Texture("./client_data/sun.jpg"));
		Object3D sunObject = Primitives.getPlane(1, 100);
		sunObject.setTexture("sun");
		sunObject.setTransparency(12);
		sunObject.setTransparencyMode(Object3D.TRANSPARENCY_MODE_ADD);
		sunObject.setLighting(Object3D.LIGHTING_NO_LIGHTS);
		sunObject.setAdditionalColor(Color.WHITE);
		sunObject.setBillboarding(true);
		sunObject.translate(0, -500, 0);
		world.addObject(sunObject);
	}
	
	private void toggleLighting() {
		if (sun.isEnabled()) {
			sun.disable();
		} else {
			sun.enable();
		}
		int worldLighting = sun.isEnabled() ? 60 : 30;
		world.setAmbientLight(worldLighting, worldLighting, worldLighting);
		Color fogColor = sun.isEnabled() ? foggingColor : Color.BLACK;
		world.setFogParameters(Config.farPlane - (sun.isEnabled() ? 100 : 200), Config.farPlane - 25, fogColor.getRed(), fogColor.getGreen(), fogColor.getBlue());

	}

	private long renderNextBlockUpdatedAt = 0L;
	private void renderNextBlock() {
		nextBlock.setVisibility(sandboxMode == 0);
		nextTile.setVisibility(sandboxMode == 1);

		if (sandboxMode == -1) {
			return;
		}
		// Get the mouse's coordinates in 2d space:
		int x = Mouse.getX();
		int y = fb.getOutputHeight() - Mouse.getY();

		// Convert that into a direction vector in world space:
		SimpleVector direction = new SimpleVector(Interact2D.reproject2D3DWS(camera, fb, x, y)).normalize();

		// Calculate the distance to whatever was clicked on:
		float distance = world.calcMinDistance(camera.getPosition(), direction, Config.farPlane);

		// Calculate the exact 3D coordinates for the point that was clicked:
		SimpleVector collisionPoint = new SimpleVector(direction);
		collisionPoint.scalarMul(distance);
		collisionPoint.add(camera.getPosition());

		int snapped_x = (int) (Math.floor(collisionPoint.x) - (Math.floor(collisionPoint.x) % 2));
		int snapped_y = (int) (Math.floor(collisionPoint.y) - (Math.floor(collisionPoint.y) % 2)) - 1;
		int snapped_z = (int) (Math.floor(collisionPoint.z) - (Math.floor(collisionPoint.z) % 2));

		nextBlock.clearTranslation();
		nextBlock.translate(snapped_x, snapped_y, snapped_z);

		nextTile.clearTranslation();
		nextTile.translate(snapped_x, snapped_y + 1, snapped_z);
	}

	private void buildBlock(boolean remove) {
		// Get the mouse's coordinates in 2d space:
		int x = Mouse.getX();
		int y = fb.getOutputHeight() - Mouse.getY();

		// Convert that into a direction vector in world space:
		SimpleVector direction = new SimpleVector(Interact2D.reproject2D3DWS(camera, fb, x, y)).normalize();

		// Calculate the distance to whatever was clicked on:
		float distance = world.calcMinDistance(camera.getPosition(), direction, Config.farPlane);

		// Check if we clicked on something:
		if (distance == Object3D.COLLISION_NONE) {
			// Nope, didn't click on anything.
		} else {
			// Calculate the exact 3D coordinates for the point that was clicked:
			SimpleVector collisionPoint = new SimpleVector(direction);
			collisionPoint.scalarMul(distance);
			collisionPoint.add(camera.getPosition());

			int snapped_x = (int) (Math.floor(collisionPoint.x) - (Math.floor(collisionPoint.x) % 2));
			int snapped_y = (int) (Math.floor(collisionPoint.y) - (Math.floor(collisionPoint.y) % 2)) - 1;
			int snapped_z = (int) (Math.floor(collisionPoint.z) - (Math.floor(collisionPoint.z) % 2));

			// placing block
			if (remove) {
				Object[] collisionObject = world.calcMinDistanceAndObject3D(camera.getPosition(), direction, Config.farPlane);
				if (collisionObject[1] != (Object) Object3D.COLLISION_NONE && collisionObject[1] != null) {
					if (collisionObject[1] instanceof TileBlock) {
						TileBlock tileBlock = (TileBlock) collisionObject[1];
						Packet packet = new Packet(5);
						packet.putInt(tileBlock.isBlock() ? 1 : 0);
						packet.putInt(tileBlock.getUid());
						packet.putFloat(snapped_x);
						packet.putFloat(snapped_y);
						packet.putFloat(snapped_z);
						connection.sendPacket(packet);
					}
				}
			} else {
				boolean isBlock = !nextTile.getVisibility();
				Packet packet = new Packet(3);
				packet.putInt(isBlock ? 1 : 0);
				packet.putInt(textureId);
				packet.putFloat(snapped_x);
				packet.putFloat(snapped_y);
				packet.putFloat(snapped_z);
				connection.sendPacket(packet);
			}
		}
	}

	public TileBlockManager getTileBlockManager() {
		return tileBlockManager;
	}

	public PlayerManager getPlayerManager() {
		return playerManager;
	}

	public void updateAnimation(int uid, int animationId) {
		playerManager.updateAnimation(player, uid, animationId);
	}

	public void updatePlayerPosition(int uid, float x, float y, float z, float rotationY) {
		playerManager.updatePosition(player, uid, x, y, z, rotationY);

	}

}
