package client.game;

import java.awt.Color;

import com.threed.jpct.Loader;
import com.threed.jpct.Matrix;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureManager;

/**
 * Here we build the 3D models, and store them in memory. And share the compiled data to reduce overhead.
 */
@SuppressWarnings("unused")
public class Cache {

	private final Object3D ranger;
	private final Object3D block;
	private final Object3D tile;

	public Cache() {
		TextureManager TM = TextureManager.getInstance();
		TM.addTexture("green", new Texture(1, 1, Color.GREEN));
		TM.addTexture("red", new Texture(1, 1, Color.RED));
		TM.addTexture("blue", new Texture(1, 1, Color.BLUE));
		TM.addTexture("cyan", new Texture(1, 1, Color.CYAN));
		TM.addTexture("white", new Texture(1, 1, Color.WHITE));
		TM.addTexture("black", new Texture(1, 1, Color.BLACK));
		TM.addTexture("block-0", new Texture("./client_data/grass.png"));
		TM.addTexture("block-1", new Texture("./client_data/block/dirt.png"));
		TM.addTexture("block-2", new Texture("./client_data/block/sand.png"));
		TM.addTexture("block-3", new Texture("./client_data/block/gravel.png"));
		TM.addTexture("block-4", new Texture("./client_data/block/stone.png"));
		TM.addTexture("block-5", new Texture("./client_data/block/cobblestone.png"));
		TM.addTexture("block-6", new Texture("./client_data/block/brick.png"));
		TM.addTexture("block-7", new Texture("./client_data/block/snow.png"));
		TM.addTexture("block-8", new Texture("./client_data/block/clay.png"));
		TM.addTexture("block-9", new Texture("./client_data/block/iron_ore.png"));
		TM.addTexture("block-10", new Texture("./client_data/block/coal_ore.png"));
		TM.addTexture("block-11", new Texture("./client_data/block/gold_ore.png"));
		TM.addTexture("block-12", new Texture("./client_data/block/planks_acacia.png"));
		TM.addTexture("block-13", new Texture("./client_data/block/planks_big_oak.png"));
		TM.addTexture("block-14", new Texture("./client_data/block/planks_oak.png"));
		TM.addTexture("block-15", new Texture("./client_data/block/planks_spruce.png"));
		for (int i = 0; i < 15; i++) {
			TM.getTexture("block-" + i).setGLFiltering(false);
		}
		
		this.ranger = Loader.loadMD2("./client_data/ranger.md2", .20f);
		ranger.compile(true);

		block = new Object3D(12);
		final int scale = 1;
		SimpleVector upperLeftFront = new SimpleVector(-scale, -scale, -scale);
		SimpleVector upperRightFront = new SimpleVector(scale, -scale, -scale);
		SimpleVector lowerLeftFront = new SimpleVector(-scale, scale, -scale);
		SimpleVector lowerRightFront = new SimpleVector(scale, scale, -scale);
		SimpleVector upperLeftBack = new SimpleVector(-scale, -scale, scale);
		SimpleVector upperRightBack = new SimpleVector(scale, -scale, scale);
		SimpleVector lowerLeftBack = new SimpleVector(-scale, scale, scale);
		SimpleVector lowerRightBack = new SimpleVector(scale, scale, scale);
		block.addTriangle(upperLeftFront, 0, 0, lowerLeftFront, 0, scale, upperRightFront, scale, 0);
		block.addTriangle(upperRightFront, scale, 0, lowerLeftFront, 0, scale, lowerRightFront, scale, scale);
		block.addTriangle(upperLeftBack, 0, 0, upperRightBack, scale, 0, lowerLeftBack, 0, scale);
		block.addTriangle(upperRightBack, scale, 0, lowerRightBack, scale, scale, lowerLeftBack, 0, scale);
		block.addTriangle(upperLeftBack, 0, 0, upperLeftFront, 0, scale, upperRightBack, scale, 0);
		block.addTriangle(upperRightBack, scale, 0, upperLeftFront, 0, scale, upperRightFront, scale, scale);
		block.addTriangle(lowerLeftBack, 0, 0, lowerRightBack, scale, 0, lowerLeftFront, 0, scale);
		block.addTriangle(lowerRightBack, scale, 0, lowerRightFront, scale, scale, lowerLeftFront, 0, scale);
		block.addTriangle(upperLeftFront, 0, 0, upperLeftBack, scale, 0, lowerLeftFront, 0, scale);
		block.addTriangle(upperLeftBack, scale, 0, lowerLeftBack, scale, scale, lowerLeftFront, 0, scale);
		block.addTriangle(upperRightFront, 0, 0, lowerRightFront, 0, scale, upperRightBack, scale, 0);
		block.addTriangle(upperRightBack, scale, 0, lowerRightFront, 0, scale, lowerRightBack, scale, scale);
		block.compile();

		this.tile = Primitives.getBox(1, .1f);
		tile.rotateY((float)Math.PI / 4);
		tile.setLighting(Object3D.LIGHTING_NO_LIGHTS);
		tile.compile();
	}

	public Object3D getRangerModel() {
		Object3D model = new Object3D(ranger, true);
		model.shareCompiledData(ranger);
		return model;
	}

	public Object3D getTileBlockModel(boolean isBlock, int type) {
		if (isBlock) {
			Object3D model = new Object3D(block, true);
			model.shareCompiledData(block);
			model.setTexture("block-" + type);
			return model;
		} else {
			Object3D model = new Object3D(tile, true);
			model.shareCompiledData(tile);
			model.setTexture("block-" + type);
			return model;
		}
	}

}