package client.net;

import java.net.InetSocketAddress;
import java.util.concurrent.Executors;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.packet.Packet;

import org.jboss.netty.bootstrap.ClientBootstrap;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.socket.nio.NioClientSocketChannelFactory;

import client.game.Application;

/**
 * Handle the connectation with the server.
 */
public class Connection {

	private Channel channel = null;
	private Application application = null;

	/**
	 * Create a new connectation instance.
	 *
	 * @param application the main client class
	 */
	public Connection(Application application) {
		this.application = application;
	}

	/**
	 * Connect the client to the server.
	 */
	public void connect(String hostname, int port) {
		try {
			ClientBootstrap bootstrap = new ClientBootstrap(new NioClientSocketChannelFactory(Executors.newCachedThreadPool(), Executors.newCachedThreadPool()));
			bootstrap.setPipelineFactory(new ClientPipelineFactory(application));
			channel = bootstrap.connect(new InetSocketAddress(hostname, port)).awaitUninterruptibly().getChannel();
		} catch (Exception e) {
			Logger.getLogger(Connection.class.getName()).log(Level.SEVERE, "No connection to the gameserver!", e);
		}
	}

	/**
	 * Disconnect the client from the server.
	 */
	public void disconnect() {
		//Check that the channel is not null (the player tired to login)
		if (channel != null) {
			channel.close();
		}
	}

	/**
	 * Return that the channel is connected.
	 *
	 * @return is connected
	 */
	public boolean isConnected() {
		if (channel != null) {
			return channel.isConnected();
		}
		return false;
	}

	/**
	 * Send a packet over the connectation.
	 *
	 * @param packet the packet
	 */
	public void sendPacket(Packet packet) {
		if (channel.isConnected()) {
			channel.write(packet);
		}
	}
}
