package com.jpct.twod;


import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import com.threed.jpct.FrameBuffer;

/**
 * 2D utilities that make rendering text and images easier for beginner jPCT users.
 */
public class TwoD {

	// 2D data
	private TexturePack texturePack;
	private String spriteIndex[];

	/**
	 * Loads all images from the given <loadSpritesFrom> path, and pack them
	 * as textures that can be rendered using <drawImage>.
	 */
	public TwoD(String loadSpritesFrom) {
		if (!loadSpritesFrom.endsWith("/")) {
			loadSpritesFrom = loadSpritesFrom + "/";
		}
		try {
			this.texturePack = new TexturePack();
			this.spriteIndex = getSpriteIndex(loadSpritesFrom);
			for (int i = 0; i < spriteIndex.length; i++) {
				try {
					File file = new File(loadSpritesFrom + spriteIndex[i]);
					texturePack.addImage(ImageIO.read(file));
				} catch (IOException e) {
					e.printStackTrace();
					System.err.println("Unable to pack sprite: " + spriteIndex[i]);
					System.exit(0);
				}
			}
			if (spriteIndex.length > 0) {
				texturePack.pack(true);
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Unable to load sprites from: " + loadSpritesFrom);
			System.exit(1);
		}
	}

	/**
	 * Draws an image. String <sprite> will be fetched from the local sprite index, and rendered
	 * at x, y, with optional transparency <transparent>
	 */
	public void drawImage(FrameBuffer frameBuffer, String sprite, int x, int y, boolean transparent) {
		try {
			texturePack.blit(frameBuffer, getSpriteId(sprite), x, y, transparent);
		} catch (Exception e) {
			System.err.println("Error drawing sprite: " + sprite);
			System.exit(1);
		}
	}

	/**
	 * Loads all images from the given <loadSpritesFrom> path,
	 * and store the filename in an arrow (minus the file extension).
	 * Example: "cursor.png" would be "cursor"
	 */
	private String[] getSpriteIndex(String loadSpritesFrom) {
		File file = new File(loadSpritesFrom);
		if (file.isDirectory()) {
			File images[] = file.listFiles();
			String sprites[] = new String[images.length];
			for (int i = 0; i < images.length; i++) {
				sprites[i] = images[i].getName();//trim(images[i].getName());
			}
			return sprites;
		}
		return null;
	}

	private int getSpriteId(String sprite) {
		sprite = trim(sprite);
		String cachedSprite;
		for (int i = 0; i < spriteIndex.length; i++) {
			cachedSprite = trim(spriteIndex[i]);
			if (cachedSprite.equalsIgnoreCase(sprite)) {
				return i;
			}
		}
		System.err.println("Sprite '" + sprite + "' does not exist!");
		System.exit(1);
		return -1;
	}

	private String trim(String string) {
		if (string.endsWith(".png")) {
			string = string.replaceAll(".png", "");
		}
		if (string.endsWith(".jpg")) {
			string.replaceAll(".jpg", "");
		}
		if (string.endsWith(".gif")) {
			string.replaceAll(".gif", "");
		}
		if (string.endsWith(".bmp")) {
			string.replaceAll(".bmp", "");
		}
		return string;
	}

}